//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file		HigherSkills.cpp
 *  \brief		Implementacia HigherSkills - vyssie schopnosti hraca
 *  \author		Rado Kostelnik
 *  \date		1.12.2001 17:28:47
 *  \version	0.1

 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
#include "stdafx.h"
#include "skillscommon.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHigherSkills::CHigherSkills()
{
	char tmp[1024];
	string fname;
	GetCurrentDirectory( 1024, tmp );

	fname = tmp;
	fname += "\\rules.xml";

	m_ruleengine.LoadXML( fname );
}

//------------------------------------------------------
/*! \fn		CHigherSkills::CHigherSkills( string sFileName )
 *  \brief	Konstruktor s parametrom - nazov suboru, ktory sa ma precitat
 *  \param  string sFileName
 *  \return
 *  \author Rado Kostelnik
 *  \date	9.12.2001 22:02:41

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CHigherSkills::CHigherSkills( string sFileName )
{
	m_ruleengine.LoadXML( sFileName );
}

CHigherSkills::~CHigherSkills()
{

}

//------------------------------------------------------
/*! \fn			void CHigherSkills::deliberate( CLocalAgent* p )
 *  \brief		Rozhodovanie o nasledujucej cinnosti na vyssej urovni
 *  \param		CLocalAgent* p agent, ktoremu urcujeme cinnosti
 *  \return		nic
 *  \author		Rado Kostelnik
 *  \date		1.12.2001 17:16:09

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
void CHigherSkills::Deliberate( CLocalAgent* p )
{
	TActionVector* av;
	CAbstractBasicSkillsList& sl = (CAbstractBasicSkillsList&) p->listSkl;  
	CAbstractBasicSkills* s = NULL;
	TParamVector v;
	int priority = 0;

	CQuery q;

	av = m_ruleengine.Evaluate( "default", q );

	if ( av )
	{
		TActionVectorIterator ai = av->begin();
		TActionVectorIterator ae = av->end();

		for ( ; ai != ae; ai++ )
		{
			CAction& a = *ai;
			//
			// Vyhodnotenie realnych hodnot parametrov akcii
			//
			TRuleParameterVectorIterator pi = a.parameters().begin();
			TRuleParameterVectorIterator pe = a.parameters().end();

			v.clear();
			for ( ; pi != pe; pi++ )
			{
				CRuleParameter& p = *pi;
				string tmps;
				long pos;

				// location
				if ( p.type == "location" )
				{
					tmps = q.getValue( p.name );
					pos = tmps.find( " " );
					v.push_back( CParam( VALUE_X, tmps.substr( 0, pos ) ) );
					v.push_back( CParam( VALUE_Y, tmps.substr( pos+1 ) ) );
				}
				// playerid
				else if ( p.type == "playerid" )
				{
					tmps = q.getValue( p.name );
					v.push_back ( CParam( VALUE_PLAYER, tmps ) );
				}
				// effort
				else if ( p.type == "effort" )
				{
					if ( p.value == "Maximum" )
						priority = 3;
					else if ( p.value == "Medium" )
						priority = 2;
					else if ( p.value == "Minimum" )
						priority = 1;
				}
				// angle
				else if ( p.type == "angle" )
				{
					tmps = q.getValue( p.name );
					v.push_back ( CParam( VALUE_ANGLE, tmps ) );
				}
				else
					cout << "Error: Unknown type: " << p.type << endl;
			}

			//
			// Kick
			//
			if ( a.name == "Kick" )
			{
				s = new CKick( v, p, priority );
			}
			else
			//
			// GoToPosition
			//
			if ( a.name == "GoToPosition" )
			{
				s = new CMove( v, p, priority );
			}
			else 
			//
			// DribbleTo
			//
			if ( a.name == "DribbleTo" )
			{
				s = new CDribble( v, p, priority );
			}
			else 
			//
			// TurnTo
			//
			if ( a.name == "TurnTo" )
			{
				s = new CTurn( v, p, priority );
			}
			else
			//
			// PassTo
			//
			if ( a.name == "PassTo" )
			{
				s = new CPass( v, p, priority );
			}
			else 
			//
			// Catch
			//
			if ( a.name == "Catch" )
			{
				s = new CCatch( v, p, priority );
			}

			if ( s )
				sl.AddSkill( s );
				
			s = NULL;
		
		}
	}
}